import React from 'react';
import Isvg from 'react-inlinesvg';
import {connect} from "react-redux";
import {Mutation, Query} from 'react-apollo'
import {withRouter} from 'react-router-dom'
import {GET_AGENTS_QUERY, GET_CLUSTERS_QUERY, POST_DISCOVER} from '../../queries'
import SectionHeader from './SectionHeader';
import {Field, formValueSelector, reduxForm, stopSubmit, SubmissionError} from 'redux-form';
import {InputField} from '../forms/forms';
import Validator from 'validatorjs';
import ButtonWithError from '../widgets/ButtonWithError';
import LoadingBar from '../widgets/LoadingBar';
import {SelectableTable} from "../table/SearchableSelectableTable";
import {AGENT_NAME, AGENT_STATUS} from "../../utils/consts";
import _ from "lodash";
import InfoPopover from "../reactPopover/InfoPopover";
import classNames from "classnames"
import localStorage from 'local-storage';
const ipaddr = require('ipaddr.js');
require("./discover.scss");

const ROW_HEIGHT = 90;

class DiscoverComponent extends React.Component {

    constructor(props, context) {
        super(props, context);
        const isDemo = window.location.href.indexOf("demo") !== -1;
        this.state = {
            selectedAgent: null,
            mutationError: "",
            isDemo
        };
    }

    submit(postDiscoverFunc, values) {
        if (!this.state.selectedAgent) {
            const error = {_error: `In order to continue please select a ${AGENT_NAME}`};
            throw new SubmissionError(error);
        }

        this.clearError();

        return postDiscoverFunc({
            variables: {
                clusterAddress: values.clusterIP,
                clusterUsername: values.clusterUsername,
                clusterPassword: values.clusterPassword,
                agentId: this.state.selectedAgent.agentId
            }
        });
    }

    continue() {
        localStorage.remove("clusterInfo");
        this.props.history.push('/clusters');
    }

    goToAddAgent(accountId) {
        const {clusterIP, clusterUsername, clusterPassword} = this.props;
        localStorage.set("clusterInfo", {clusterIP: clusterIP, clusterUsername: clusterUsername, clusterPassword: clusterPassword});
        const url = process.env.REACT_APP_SAAS_URL;
        window.location = `${url}?serviceName=cloudTiering&accountId=${accountId}&redirect=${window.location.href}`;
    }

    clearError() {
        this.setState({mutationError: ""});
    }

    handleMutationError(error) {
        let theError = null;
        if (error.graphQLErrors && error.graphQLErrors.length > 0) {
            theError = error.graphQLErrors[0].message;
        } else {
            theError = _.get(error, "networkError.result.errors[0].message", null);
        } //to fix TIERING-270 clusterIp should have validation on server

        theError = theError || "Unknown Error";

        if (theError !== this.state.mutationError) {
            this.setState(Object.assign({}, this.state, {mutationError: theError}));
        }
    }

    updatePageSize(data) {
        const numOfAgents = _.get(this.state, "numOfAgents", null);
        if (data && numOfAgents !== data.agents.agents.length) {
            this.setState({numOfAgents: data.agents.agents.length});
        }
    }

    renderRow(agent) {
        const descrPart1 = `${agent.provider === "AWS" ? "VPC" : "VNET"}: ${agent.network}`;
        const descrPart2 = `Subnet: ${agent.subnet}`;
        return (
            <div className="table-rows ">
                <td className="check"><Isvg cacheGetRequests={true}
                                            src={require("./../../assets/svgs/checkmark-fill-circle-icon-discover.svg")}></Isvg>
                </td>
                <td className="agent">
                    <div className={classNames("agent-name", {inactive: agent.status !== AGENT_STATUS.active.status})}>

                        {agent.status === AGENT_STATUS.active.status &&
                            <div className="name no-wrap" title={agent.name}>
                                <Isvg cacheGetRequests={true} src={require("./../../assets/svgs/cluster-status-green-old.svg")}/>
                                {agent.name}
                            </div>    }
                        {agent.status !== AGENT_STATUS.active.status &&
                            <InfoPopover textElement={
                                <div className="name float-right">
                                    <div className={`no-wrap float-left not-active ${agent.status}`} title={agent.name}>{agent.name}</div>
                                    {agent.status === AGENT_STATUS.pending.status &&
                                    <span className="italic pending-text">Pending</span>}
                                </div>}
                                 popoverClassName={`discover ${agent.status}`} wrapperClassName={agent.status}
                                 popoverText={agent.reason} contentStyle={{width: "300px"}}/>}
                        <div className="agent-description"><div className="no-wrap" title={descrPart1}>{descrPart1}</div><div>|</div><div className="no-wrap" title={descrPart2}>{descrPart2}</div></div>
                    </div>
                </td>
            </div>
        )
    }

    render() {
        const {handleSubmit, error: formError} = this.props;

        document.body.className = "no-sso discover"; //to remove the widget
        const discoverTitle = "Discover ONTAP Cluster";

        const getStyleObject = (numOfAgents) => {
            if (numOfAgents) {
                if (numOfAgents <= 3) return {};
                else if (numOfAgents > 3 && numOfAgents <= 5) {
                    return {height: "700px"};
                }
                else {
                    return {height: `${numOfAgents * ROW_HEIGHT}px`}
                }
            }
            return {};
        };

        const styleObject = getStyleObject(this.state.numOfAgents);


        return (
            <div className="discover">
                <div className="discover-header">
                    <SectionHeader title={discoverTitle} svgPath={require("./../../assets/svgs/discover.svg")}
                                   back="/"/>
                </div>
                <div className="discover-body-wrapper">
                    <Mutation mutation={POST_DISCOVER} refetchQueries={[{query: GET_CLUSTERS_QUERY}]}
                              awaitRefetchQueries={false} onCompleted={this.continue.bind(this)}
                              onError={this.handleMutationError.bind(this)} errorPolicy="all">
                        {(postDiscover, {loading}) => {
                            return (
                                <form className="discover-form"
                                      onSubmit={handleSubmit(this.submit.bind(this, postDiscover))}>
                                    <div className="discover-form-inputs"
                                         style={styleObject}>
                                        <div className="cluster-info">
                                            <div className="discover-form-header">
                                                <Isvg src={require('./../../assets/svgs/discover-circle-1.svg')}/>
                                                <h1 className="header-text">Cluster ONTAP info</h1>
                                            </div>
                                            <div className="form-description">
                                                Please fill the details of the cluster you would like to tier:
                                            </div>
                                            <div className="cluster-info-fields">
                                                <div className="cluster-ip">
                                                    <Field name="clusterIP" component={InputField}
                                                           label="Cluster Management IP Address"
                                                           type="text"></Field>
                                                </div>
                                                <div className="cluster-username">
                                                    <Field name="clusterUsername" component={InputField}
                                                           label="User Name"
                                                           type="text"></Field>
                                                </div>
                                                <div className="cluster-password">
                                                    <Field name="clusterPassword" component={InputField}
                                                           label="Password"
                                                           type="password"></Field>
                                                </div>
                                            </div>
                                        </div>

                                        <div className="vertical-line"
                                             style={styleObject}></div>

                                        <div className="connect-instances"
                                             style={styleObject}>
                                            <div className="discover-form-header">
                                                <Isvg src={require('./../../assets/svgs/discover-circle-2.svg')}/>
                                                <h1 className="header-text">{AGENT_NAME}</h1>
                                            </div>
                                            <div className="form-description">
                                                In order to discover the cluster you must have connectivity to the
                                                cluster network.
                                                Select a {AGENT_NAME} that has access to the cluster network or create a
                                                new one.
                                            </div>
                                            <div className="connect-instances-options">
                                                <Query query={GET_AGENTS_QUERY} pollInterval={30 * 1000}
                                                       onError={this.handleMutationError.bind(this)}
                                                       onCompleted={this.updatePageSize.bind(this)}>
                                                    {({loading, data}) => {
                                                        if (loading || !data) return <LoadingBar
                                                            className="relative"></LoadingBar>;
                                                        const {agents} = data;
                                                        // agents.agents = this.mockAgentsData();
                                                        const agentsWithProperties = _.map(agents.agents, agent => {
                                                            return {
                                                                isDisabled: agent.status !== AGENT_STATUS.active.status,
                                                                order: AGENT_STATUS[agent.status].order,
                                                                ...agent
                                                            }
                                                        });

                                                        return (
                                                            <div className="agents">
                                                                <div className="mask"></div>
                                                                <div className={`agent-item add-agent ${this.state.isDemo ? 'disabled' : ''}`}
                                                                     title={this.state.isDemo ? "Adding a Service Connector is not supported in the demo environment." : ""}
                                                                     onClick={() => this.state.isDemo ? _.noop() : this.goToAddAgent(agents.accountId)}>
                                                                    <Isvg
                                                                        src={require('./../../assets/svgs/ic-plus.svg')}/>
                                                                    <span className="text">Add {AGENT_NAME}</span>
                                                                </div>

                                                                {agentsWithProperties.length > 0 &&
                                                                <SelectableTable Table={{
                                                                                     renderRow: this.renderRow.bind(this)
                                                                                 }}
                                                                                 data={_.sortBy(agentsWithProperties, ['order'])}
                                                                                 selectionOptions={{
                                                                                     selector: this.handleSelect.bind(this),
                                                                                     selectedRows: this.state.selectedAgent ? [this.state.selectedAgent] : [],
                                                                                     getKey: this.getKey.bind(this)
                                                                                 }}
                                                                />}
                                                            </div>
                                                        )
                                                    }}
                                                </Query>
                                            </div>
                                        </div>

                                    </div>
                                    <div className="wizard-button">
                                        <ButtonWithError submitting={loading} className="button secondary"
                                                         type="submit" error={this.state.mutationError || formError}>
                                            Continue
                                        </ButtonWithError>
                                    </div>
                                </form>
                            )
                        }}
                    </Mutation>
                </div>
            </div>
        )
    }

    handleSelect(selectedAgent) {
        this.props.dispatch(stopSubmit("discover", {})); //clear errors
        const selectedAgentKey = this.getKey(selectedAgent);
        this.setState({selectedAgent: selectedAgentKey});
    }

    getKey(agent) {
        return _.pick(agent, ["agentId"]);
    }
}


const rules = {
    clusterIP: 'required',
    clusterUsername: 'required',
    clusterPassword: 'required'
};

const validate = (values) => {
    let errors = {};
    const validator = new Validator(values, rules);
    validator.setAttributeNames({
        clusterIP: "Cluster Management IP Address",
        clusterUsername: "Cluster User Name",
        clusterPassword: "Cluster Password",
        selectedAgent: AGENT_NAME
    });
    validator.passes();
    errors = validator.errors.all();
    if (values.clusterIP && ((values.clusterIP.length < 7) || (values.clusterIP && values.clusterIP.indexOf('0.0.0.0') > -1) || !(ipaddr.isValid(values.clusterIP)))) {
        const clusterIpError = ["Invalid Cluster Management IP Address"];
        if (errors.clusterIP) {
            errors.clusterIP.push(clusterIpError)
        }
        else {
            errors.clusterIP = clusterIpError;
        }
    }
    return errors;
};

const Discover = reduxForm({
    form: 'discover',
    validate
})(DiscoverComponent);

const selector = formValueSelector('discover');
const ConnectedDiscover = connect(state => {
    const clusterInfo = localStorage.get("clusterInfo") || {clusterIP: "", clusterUsername: "", clusterPassword: ""};

    const clusterIP = selector(state, 'clusterIP');
    const clusterUsername = selector(state, 'clusterUsername');
    const clusterPassword = selector(state, 'clusterPassword');

    return {
        initialValues: {
            clusterIP : clusterInfo.clusterIP,
            clusterUsername: clusterInfo.clusterUsername,
            clusterPassword: clusterInfo.clusterPassword
        },
        clusterIP, clusterUsername, clusterPassword
    };
})(Discover);
export default withRouter(ConnectedDiscover);
