import React, {Component} from 'react';
import {Link} from "react-router-dom";
import Isvg from "react-inlinesvg";
import _ from "lodash";
import {DASHBOARD_TITLES, PROVIDERS, TIER_BUTTON_TEXT} from "../../utils/consts";
import {CardItem, ValuesContainer} from "./CardComponents";
import {Doughnut} from 'react-chartjs-2';
import {calculateSavings, buildCapacityObject} from "../../utils/utils";
import {Estimated} from "./CardComponents";
import classNames from 'classnames'
import InfoPopover from "../reactPopover/InfoPopover";
import localStorage from "local-storage";
import {Query} from "react-apollo";
import {GET_CLUSTERS_QUERY} from "../../queries";
import LoadingIcon from "../widgets/LoadingIcon";

require("./moreInfo.scss");

export const InfoHeader = (props) => {
    const {match: {params: {clusterId}}, data: {clusters}} = props;
    const selectedCluster = _.find(clusters, {clusterId});
    const clusterName = selectedCluster ? selectedCluster.name : "";
    return <div className="more-info-header">
        <div className="name">{clusterName}</div>
        <Link className="close" to={`/clusters`}><Isvg cacheGetRequests={true}
                                                       src={require("./../../assets/svgs/x.svg")}></Isvg></Link>
    </div>
};

const InsideGraphInfo = ({info: {insideGraph: {value, unit, subTitle}}, icon}) => {
    return (
        <div className="item">
            <div className="icon">{icon}</div>
            <div className="size">{value}{unit && <span>{` ${unit}`}</span>}</div>
            <div className="sub-title">{subTitle}</div>
        </div>
    );
};

const BottomInfoItem = ({item}) => {
    return <div className="bottom-item">
        <div className="size">{item.value}{item.unit && <span>{` ${item.unit}`}</span>}</div>
        <div className="rectangle float-left" style={{"backgroundColor": item.color}}/>
        <div className="bottom-title float-left">{item.bottomTitle}</div>
    </div>
};

class InfoGraphCard extends Component {
    constructor(props, context) {
        super(props, context);
        this.state = {showGraph: false};
    }

    componentDidMount() {
        const {delay} = this.props;
        return setTimeout(() => this.setState({showGraph: true}), delay);
    }

    render() {
        const {insideGraph, icon, bottomInfoItems, estimated, primaryTitle, children, className} = this.props;
        const {showGraph} = this.state;
        return <div className={`info-graph-card ${className}`}>
            <div className="primary title">{primaryTitle}
                {estimated && <Estimated text={"Estimated Values"} popoverText={DASHBOARD_TITLES.estimated}/>}
            </div>
            <div className="graph">
                {showGraph &&
                <Doughnut options={{preserveAspectRatio: false, cutoutPercentage: 85, tooltips: {enabled: false}}}
                          legend={{display: false}} data={{
                    labels: bottomInfoItems.map(item => item.bottomTitle),
                    datasets: [{
                        data: bottomInfoItems.map(item => item.bytes),
                        backgroundColor: bottomInfoItems.map(item => item.color)
                    }]
                }
                }/>}

                <div className="inside-graph">
                    {insideGraph && <InsideGraphInfo info={insideGraph} icon={icon}/>}
                </div>
                <div className="bottom">
                    {bottomInfoItems.map((bottomItem, index) => {
                        return <BottomInfoItem item={bottomItem} key={index}/>
                    })}
                </div>
            </div>
            <div className="gray-info title">
                {children}
            </div>
        </div>

    }
}

const MoreInfoRow = ({row}) => {
    return <div className="info-row">
        {!row.popoverText && <div className="info-key">{row.key}</div>}
        {row.popoverText &&
        <InfoPopover trigger={"mouseover-exit-click"} wrapperClassName="info-key" popoverClassName="info-key"
                     textElement={row.key} popoverText={row.popoverText} contentStyle={{width: "545px"}}/>}
        <div className={classNames("info-value", {"no-wrap": !row.icon})}
             title={row.icon ? "" : row.value}>{row.icon ? row.icon : row.value}</div>
    </div>
};

const MoreInfoCard = ({title, infoRows, noInfoRowsText, className}) => {
    return <div className={`more-info-card ${className}`}>
        <div className="title">{title}</div>
        {infoRows && <div className="rows-container">
            {infoRows.map((row, index) => {
                return <MoreInfoRow row={row} key={index}/>
            })}
        </div>}
        {noInfoRowsText && <div className="no-info italic">{noInfoRowsText}</div>}
    </div>
};

export default class MoreInfo extends Component {
    componentDidMount() {
        //since we can now return from wizard to MoreInfo we need to clear all wizard params kept on the local storage here as well. and not only in dashboard
        localStorage.remove("selectedVolumes");
        localStorage.remove("selectedAwsKeys");
    }

    handleError(error) {
        this.props.history.push('/');
    }

    render() {
        const {match: {params: {clusterId}}} = this.props;

        return <Query query={GET_CLUSTERS_QUERY} variables={{clusterId}} fetchPolicy="network-only" onError={this.handleError.bind(this)}>
            {({loading, error, data}) => {
                if (loading) {
                    return (<LoadingIcon className="loading-clusters" text={"Cluster's Info"} icon={<Isvg
                        src={require('../../assets/svgs/review-cluster.svg')}/>}/>);
                }
                if (error) {
                    this.handleError();
                    return <div></div>;
                }
                else {
                    const cluster = data.clusters ? data.clusters[0] : [];
                    if (cluster.status.status === "UNKNOWN" || cluster.status.status === "FAILED") {
                        this.handleError();
                        return <div></div>;
                    }

                    const {volumes: {eligibleVolumes, tieredVolumes}, ontapClusterProperties, tierInformation, capacityInformation} = cluster;

                    const {currentSavings, additionalSavings} = calculateSavings(cluster);

                    const clusterInfo = ontapClusterProperties ? [
                        {key: "IP", value: ontapClusterProperties.clusterAddress},
                        {key: "ONTAP Version", value: ontapClusterProperties.ontapVersion},
                        {
                            key: "Tiered Volumes", value: `${tieredVolumes} / ${eligibleVolumes}`,
                            popoverText: <span>The total number of volumes that Cloud Tiering displays might be less than the number of volumes on your ONTAP system. That's because some volumes can't be tiered. For example, you can't tier data from SnapLock volumes or from MetroCluster configurations. Refer to
                    <a href="https://docs.netapp.com/ontap-9/index.jsp?topic=%2Fcom.netapp.doc.dot-cm-psmg%2FGUID-8E421CC9-1DE1-492F-A84C-9EB1B0177807.html"
                       target="_blank" rel="noopener noreferrer"> ONTAP Documentation </a>
                    for features not supported by FabricPool.</span>
                        }
                    ] : [];

                    const cloudProvider = _.get(cluster, ["tierInformation", "providerType"], "AWS");
                    const objectStorageInfo = tierInformation ? [
                        {key: "Provider", icon: PROVIDERS[cloudProvider].icon},
                        {key: cloudProvider === "AWS" ? "S3 Bucket" : "Azure Blob", value: tierInformation.bucketName},
                        {key: "Storage Class", value: tierInformation.tierLevel}
                    ] : null;


                    const tierCapacity = buildCapacityObject(_.get(tierInformation, "tieredCapacity", 0));
                    const performanceUsedCapacity = buildCapacityObject(_.get(capacityInformation, "performanceCapacity.usedPerformanceCapacity", 0));
                    const totalCapacity = buildCapacityObject(_.get(capacityInformation, "usedPerformanceAndTieredCapacity", 0));
                    const coldData = buildCapacityObject(_.get(capacityInformation, "performanceCapacity.inactiveCapacity", 0));
                    const hotData = buildCapacityObject(_.get(capacityInformation, "performanceCapacity.activeCapacity", 0));

                    // console.log(`%c id ${clusterId} tierCapacity: ${JSON.stringify(tierCapacity)} performanceUsedCapacity ${JSON.stringify(performanceUsedCapacity)} total ${JSON.stringify(totalCapacity)} cold ${JSON.stringify(coldData)} hot ${JSON.stringify(hotData)}`, 'font-weight: bold; color:green');

                    const totalCapacityInfo = {
                        insideGraph: {
                            ...totalCapacity,
                            subTitle: "Total Used Capacity"
                        }
                    };

                    const usedCapacityInfo = {
                        insideGraph: {
                            ...performanceUsedCapacity,
                            subTitle: "Used Capacity"
                        }
                    };

                    const cameFrom = window.location.pathname;
                    const nextRoute = tieredVolumes ? `/retier/${clusterId}/volumes?cameFrom=${cameFrom}` : `/tier/${clusterId}/volumes?cameFrom=${cameFrom}`;

                    return (<div className="more-info">
                        <div className="info-cards-container">
                            <InfoGraphCard primaryTitle="Total Used Capacity" insideGraph={totalCapacityInfo}
                                           className="enter-1 total-used-capacity"
                                           icon={<Isvg cacheGetRequests={true}
                                                       src={require('../../assets/svgs/total-used-capacity.svg')}/>}
                                           delay={800}
                                           bottomInfoItems={[
                                               {
                                                   ...tierCapacity,
                                                   bottomTitle: "S3 Tiered Capacity", color: "#36bbca"
                                               },
                                               {
                                                   ...performanceUsedCapacity,
                                                   bottomTitle: "Cluster Used Capacity", color: "#3d84ff"
                                               }
                                           ]}>
                                <CardItem className="gray-card-item" title="Current Savings"
                                          icon={<Isvg cacheGetRequests={true}
                                                      src={require(`./../../assets/svgs/current-savings.svg`)}/>}>
                                    <ValuesContainer values={currentSavings}/>
                                </CardItem>
                            </InfoGraphCard>
                            <InfoGraphCard primaryTitle="Cluster Used Capacity" insideGraph={usedCapacityInfo}
                                           className="enter-2"
                                           icon={<Isvg cacheGetRequests={true}
                                                       src={require('./../../assets/svgs/cluster-used-capacity.svg')}/>}
                                           estimated={_.get(capacityInformation, "performanceCapacity.isEstimatedInactiveCapacity", false)}
                                           delay={1200}
                                           bottomInfoItems={[
                                               {
                                                   ...hotData,
                                                   bottomTitle: "Hot Data", color: "#da0075"
                                               },
                                               {
                                                   ...coldData,
                                                   bottomTitle: "Cold Data", color: "#3d84ff"
                                               }
                                           ]}>

                                <CardItem className="gray-card-item" title="Cold Data Saving Opportunity"
                                          icon={<Isvg cacheGetRequests={true}
                                                      src={require(`./../../assets/svgs/additional-savings.svg`)}/>}>
                                    <ValuesContainer values={additionalSavings}/>
                                    {eligibleVolumes > tieredVolumes && <Link to={nextRoute}>
                                        <button type="button" className="tier-button">{TIER_BUTTON_TEXT}</button>
                                    </Link>}
                                </CardItem>
                            </InfoGraphCard>
                            <div className="more-info-cards-container">
                                <MoreInfoCard title="Cluster Info" infoRows={clusterInfo} className="enter-3"/>
                                <MoreInfoCard title="Object Storage Info" infoRows={objectStorageInfo} className="enter-4"
                                              noInfoRowsText={!objectStorageInfo ? DASHBOARD_TITLES.notTiered : ""}/>
                            </div>
                        </div>
                    </div>)
                }

            }}
        </Query>
    }
}

