import React, {Fragment} from "react"
import {withRouter} from 'react-router-dom'
import InfoPopover from '../reactPopover/InfoPopover'
import Isvg from "react-inlinesvg";
import {connect} from "react-redux";
import _ from 'lodash';
import ErrorsPanel from '../errors/ErrorsPanel'
import classNames from 'classnames'
import errorsDispatcher from '../../dispatchers/errorsDispatcher'
import {
    DOCUMENTATION_URL,
    FREE_TRIAL_EXPIRED, FREE_TRIAL_WARNING, PRE_FREE_TRIAL,
    SHOW_TRIAL_INFO,
    SUBSCRIBED
} from "../../utils/consts";
import {Query} from "react-apollo";
import {GET_ACCOUNT_INFO_QUERY} from "../../queries";
import {getAccountInfoStatus} from "../../utils/utils";


require("./footer.scss");

const calculateLicenseStatusFooter = (accountInfoStatus) => {
    let licenseStatusText = 'LOADING...', status = "";

    if (accountInfoStatus) {
        const preFreeTrial = accountInfoStatus.freeTrialStatus === PRE_FREE_TRIAL;
        const trialEnded = accountInfoStatus.freeTrialStatus === FREE_TRIAL_EXPIRED;
        const subscribed = accountInfoStatus.subscriptionStatus === SUBSCRIBED;

        if (preFreeTrial) {
            licenseStatusText = "";
            status = "hide";
        }
        else if (!trialEnded) {
            licenseStatusText = `FREE TRIAL [${accountInfoStatus.daysLeftForTrial} ${accountInfoStatus.daysLeftForTrial > 1 ? 'Days' : 'Day'} left]`;
            status = accountInfoStatus.freeTrialStatus === FREE_TRIAL_WARNING && !subscribed ? "warning" : "";
        } else if (subscribed) {
            licenseStatusText = 'ACTIVE SUBSCRIPTION';
            status = "hide"
        } else {
            licenseStatusText = 'SUBSCRIPTION REQUIRED';
            status = "error";
        }
    }
    return {licenseStatusText, status};
};


const renderLicenseStatusFooter = ({licenseStatusText, status}, history) => {
    return (
        <Fragment>
            {status !== "hide" && <div className={classNames("license-status-footer", {"footer-error": status === "error", "footer-warning": status === "warning"})}>
                <button onClick={() => {
                    history.push('/license');
                }}>
                    <Isvg cacheGetRequests={true} src={require("../../assets/svgs/exclamation-mark-triangle-icon.svg")}/>
                    {licenseStatusText}
                </button>
            </div>}
            {status === "hide" && <div></div>}
        </Fragment>
    );
};

const showTrialInfo = SHOW_TRIAL_INFO;

const Footer = (props) => {
    const {errors, errorsPanelOpen} = props;
    const shouldShowErrorsIcon = errors && errors.length > 0 && !errorsPanelOpen;
    const errorsMessage = shouldShowErrorsIcon ? (errors.length === 1 ? "1 Error" : `${errors.length} Errors`) : "";
    return (<Query query={GET_ACCOUNT_INFO_QUERY} fetchPolicy="network-only" pollInterval={30*1000}>
        {({loading, data}) => {
            if (loading) return <div/>;
            const {accountInfo} = data;
            const accountInfoStatus = getAccountInfoStatus(accountInfo);
            return (
                <div>
                    <ErrorsPanel {...props}/>}
                    <footer className="footer">
                        <div className="footer-container">
                            {process.env.REACT_APP_VERSION && <div className="version">{`Version: ${process.env.REACT_APP_VERSION}`}</div>}
                            {showTrialInfo && renderLicenseStatusFooter(calculateLicenseStatusFooter(accountInfoStatus), props.history)}
                            {!showTrialInfo &&<InfoPopover textElement={<span>Controlled Availability</span>}
                                                           wrapperClassName={"controlled-availability"}
                                                           contentStyle={{width: "470px"}}
                                                           popoverText="This Controlled Availability release enables NetApp to gather feedback about Cloud Tiering from our customers. Production workloads are supported. You will not need to migrate your data when Cloud Tiering reaches General Availability."/>
                            }
                            <div className="documentation">
                                <a href={DOCUMENTATION_URL}
                                   target="_blank" rel="noopener noreferrer">
                                    <Isvg cacheGetRequests={true} src={require("./../../assets/svgs/help.svg")}/>Documentation
                                </a>
                            </div>
                            <div className={classNames("errors", {"is-visible": shouldShowErrorsIcon})}
                                 onClick={() => props.toggleErrorsState()}>
                                <Isvg cacheGetRequests={true} src={require("./../../assets/svgs/errors-footer.svg")}></Isvg>
                                {errorsMessage}
                            </div>
                        </div>
                    </footer>
                </div>
            );
        }}
    </Query>)
};

const mapStateToProps = state => {
    const errors = _.orderBy(_.values(state.errors.generalErrors.errorsMap), "time", "desc");
    return {
        errors,
        errorsPanelOpen: state.errors.generalErrors.errorsPanelOpen,
        errorsPanelSize: state.errors.generalErrors.errorsPanelSize
    }
};


export default withRouter(connect(mapStateToProps, errorsDispatcher)(Footer));