import React, {Component, Fragment} from 'react';
import {compose, graphql, Mutation} from 'react-apollo'
import {connect} from 'react-redux';
import stepinize from "./../wizard/stepinize";
import mapStateToProps from './wizardMapStateToProps'
import {ADD_TO_TIER, GET_CLUSTERS_QUERY, GET_REGIONS_QUERY, START_TIERING} from "../../queries";
import Isvg from "react-inlinesvg";
import ButtonWithError from "../widgets/ButtonWithError";
import localStorage from 'local-storage';
import _ from 'lodash'
import LoadingBar from '../widgets/LoadingBar';
import classNames from 'classnames'
import LoadingIcon from "../widgets/LoadingIcon";
import {DEFAULT_STORAGE_CLASS, STORAGE_CLASSES, TIERING_POLICY} from "../../utils/consts";
import {extractByType, extractError} from "../../utils/utils";

require('./reviewStep.scss');


const ReviewCard = ({className, cardTitle, values, icon}) => {
    return (
        <div className={`review-card ${className}`}>
            {/*<div className="overlay"></div>*/}
            <div className="card-title-container">
                <div className="icon">
                    {icon}
                </div>
                <div className="card-title">
                    {cardTitle}
                </div>
            </div>
            <div className="container">
                {values.map((value, index) => {
                    return (<div className="text" key={index}>
                        <div className="sub-title no-wrap">{value.subTitle}</div>
                        <div className="title no-wrap" title={value.title}>{value.title}</div>
                    </div>)
                })}
            </div>
        </div>
    )
};

class ReviewStep extends Component {
    constructor(props, context) {
        super(props, context);
        this.state = {tierStarted: false};
    }

    componentDidMount() {
        this.setState({tierStarted: false});
    }

    componentDidUpdate(prevProps) {
        const {getClustersQuery: {error: getClustersError}, getRegionsQuery} = prevProps;
        if (!this.state.error && (getClustersError || (getRegionsQuery && getRegionsQuery.error))) {
            this.setState({error: extractError(getClustersError || getRegionsQuery.error, "Review Step")});
        }
    }

    continue() {
        this.props.history.push('/clusters');
    }

    handleError(error) {
        this.setState({error : extractError(error)});
    }

    handleSubmit(tieringMethod, mutationVariables, event) {
        this.setState({tierStarted: true});
        event.preventDefault();
        tieringMethod({
            variables: mutationVariables
        });
    }

    getDiagramUx({clusterName, numOfVolumes, totalVolumes, tieringPolicy, ipSpace}, {regionName, storageClass, bucketName, provider}) {
        const clusterValues = [
            {
                title: clusterName,
                subTitle: "Cluster Name"
            },
            {
                title: `${numOfVolumes}/${totalVolumes}`,
                subTitle: "Selected Volumes"
            },
            {
                title: `${tieringPolicy}`,
                subTitle: "Tiering Policy"
            }
        ];
        if (ipSpace) {
            clusterValues.push({title: `${ipSpace}`, subTitle: "IPspace"});
        }
        const bucketValues = [
            {
                title: storageClass,
                subTitle: "Storage Class"
            }
        ];

        if (regionName) {
            bucketValues.unshift({
                title: regionName,
                subTitle: "Region Name"
            })
        }

        if (bucketName) {
            bucketValues.unshift({
                title: bucketName,
                subTitle: "Bucket Name"
            })
        }

        return (<div className="diagram">
            <ReviewCard className="cluster" cardTitle="Cluster" values={clusterValues}
                        icon={<Isvg cacheGetRequests={true}
                                    src={require(`./../../assets/svgs/review-cluster.svg`)}/>}></ReviewCard>
            <div className="arrows"><Isvg src={require('./../../assets/svgs/review-arrows.svg')}/></div>

            <ReviewCard className="bucket" cardTitle={provider === "AWS" ? "S3 Bucket" : "Blob Object Storage"}
                        values={bucketValues} icon={<Isvg cacheGetRequests={true}
                                                          src={require(`./../../assets/svgs/review-bucket.svg`)}/>}></ReviewCard>
        </div>)

    }

    getMutationFormUX(handleSubmitMethod) {
        const error = this.state.error;
        return (
            <div>
                <form onSubmit={handleSubmitMethod}>
                    <div className="wizard-button">
                        <ButtonWithError type="submit" className="button secondary tier" error={error}>Tier
                            Cluster</ButtonWithError>
                    </div>
                </form>
            </div>
        )
    }

    getTierUX(getRegionsQuery, getClustersQuery) {
        if (!getClustersQuery|| !getRegionsQuery) return (<div></div>);
        const {error: getRegionsError, loading: getRegionsLoading, regions} = getRegionsQuery;
        const {error: getClustersError, loading: getClustersLoading, clusters} = getClustersQuery;
        if (getRegionsError || getClustersError) return (<div></div>);
        if (getClustersLoading) { //due to skip there might not be data yet
            return (<LoadingIcon text={"Cluster"} className="loading-cluster" icon={<Isvg
                src={require('../../assets/svgs/review-cluster.svg')}/>}/>);
        }
        if (getRegionsLoading) { //due to skip there might not be data yet
            return (<LoadingIcon text={"Regions"} className="loading-regions" icon={<Isvg
                src={require('../../assets/svgs/bucket-icon.svg')}/>}/>);
        }
        //when query result arrives from the cach it arrives as an object and not as an array
        const cluster = extractByType(clusters);
        // console.log(`%c ReviewStep Cluster is ${JSON.stringify(cluster)}`, 'font-weight:bold;color:green');
        const {name, volumes: {eligibleVolumes}} = cluster;
        const {wizard: {selectedWizardParams: {clusterId, region, storageClass, provider, tieringPolicy, ipSpace}}} = this.props;
        if (!tieringPolicy || !TIERING_POLICY[tieringPolicy]) { //sometimes it takes a bit time until all the url params are there...
            return <div></div>
        }



        const volumes = _.compact(localStorage.get("selectedVolumes"));
        const awsKeys = localStorage.get("selectedAwsKeys");
        const tieringPolicyName = TIERING_POLICY[tieringPolicy].name;
        //when query result arrives from the cach it arrives as an object and not as an array
        const regionName = region ? (_.isArray(regions) ? _.find(regions, {region: region}).regionName : regions.regionName): "";
        const storageClassName = storageClass ? _.find(STORAGE_CLASSES[provider], {storageClass}).storageClassName : STORAGE_CLASSES[provider][DEFAULT_STORAGE_CLASS].storageClass;
        return (<Fragment>
            {this.getDiagramUx({
                    clusterName: name,
                    numOfVolumes: volumes.length,
                    totalVolumes: eligibleVolumes,
                    tieringPolicy: tieringPolicyName,
                    ipSpace
                },
                {
                    regionName,
                    storageClass: storageClassName,
                    provider
                })
            }
            <Mutation mutation={START_TIERING} onCompleted={this.continue.bind(this)}
                      onError={this.handleError.bind(this)} errorPolicy="all"
                      refetchQueries={[
                          {query: GET_CLUSTERS_QUERY, awaitRefetchQueries: false}
                      ]}>
                {(startTiering, {loading}) => {
                    if (loading) {
                        return <LoadingBar/>;
                    }
                    const variables = {
                        clusterId,
                        volumeIdentifiers: volumes,
                        providerProperties: {
                            cloudProvider: provider,
                            storageClass,
                            region,
                            awsKeys: provider === "AWS" ? awsKeys : undefined
                        },
                        tieringPolicy,
                        ipSpace
                    };
                    return this.getMutationFormUX(this.handleSubmit.bind(this, startTiering, variables));
                }}
            </Mutation>
        </Fragment>)
    }

    getRetierUx(getClustersQuery) {
        const {loading, error, clusters} = getClustersQuery;
        if (loading) { //due to skip there might not be data yet
            return (<LoadingIcon text={"Cluster"} className="loading-cluster" icon={<Isvg
                src={require('../../assets/svgs/review-cluster.svg')}/>}/>);
        }
        if (error) return (<div></div>);
        //when query result arrives from the cach it arrives as an object and not as an array
        const cluster = extractByType(clusters);
        const {wizard: {selectedWizardParams: {tieringPolicy}}} = this.props;
        const {name, clusterId, tierInformation: {regionName, bucketName, tierLevel, providerType}, volumes: {eligibleVolumes}} = cluster;
        if (!tieringPolicy || !TIERING_POLICY[tieringPolicy]) { //sometimes it takes a bit time until all the url params are there...
            return <div></div>
        }
        const volumes = _.compact(localStorage.get("selectedVolumes"));
        const tieringPolicyName = TIERING_POLICY[tieringPolicy].name;

        return (<Fragment>
            {this.getDiagramUx({
                    clusterName: name,
                    numOfVolumes: volumes.length,
                    totalVolumes: eligibleVolumes,
                    tieringPolicy: tieringPolicyName
                },
                {
                    regionName,
                    storageClass: tierLevel,
                    bucketName,
                    provider: providerType
                })
            }
            <Mutation mutation={ADD_TO_TIER} onCompleted={this.continue.bind(this)}
                      onError={this.handleError.bind(this)} errorPolicy="all"
                      refetchQueries={[
                          {query: GET_CLUSTERS_QUERY, awaitRefetchQueries: false}
                      ]}>
                {(addToTier, {loading}) => {
                    if (loading) {
                        return <LoadingBar/>;
                    }
                    const variables = {
                        clusterId,
                        volumeIdentifiers: volumes,
                        tieringPolicy
                    };
                    return this.getMutationFormUX(this.handleSubmit.bind(this, addToTier, variables));
                }}
            </Mutation>
        </Fragment>)
    }

    render() {
        if (!this.props.wizard || !this.props.wizard.selectedWizardParams || this.props.getClustersQuery.loading) {
            return <div></div> //sometimes it takes a bit time until all the url params are there...
        }
        const {up, wizard: {wizardType}, getClustersQuery, getRegionsQuery} = this.props;
        return (
            <div className={classNames("review-step wizard-step", {
                up,
                down: !up,
                "tier-started": this.state.tierStarted
            })}>
                {/*<div className="background"></div>*/}
                <div className="step-title">
                    <Isvg src={require("./../../assets/svgs/review-icon.svg")}/>
                    <h3>Review & Approve</h3>
                </div>
                {wizardType === "tier" && this.getTierUX(getRegionsQuery, getClustersQuery)}
                {wizardType === "retier" && this.getRetierUx(getClustersQuery)}
            </div>
        )
    }
}

export default compose(
    connect(mapStateToProps),
    graphql(GET_CLUSTERS_QUERY, {
        name: "getClustersQuery",
        skip: props => {
            const {routingInfo: {urlParams: {decodedParams: {clusterId}}}} = props;
            return !clusterId;
        },
        options: (props) => {
            const {routingInfo: {urlParams: {decodedParams: {clusterId}}}} = props;
            return {
                variables: {
                    clusterId
                },
                fetchPolicy: "cache-first"
            };
        }
    }),
    graphql(GET_REGIONS_QUERY, {
        name: "getRegionsQuery",
        skip: props => {
            const {routingInfo: {urlParams: {decodedParams: {clusterId, region, provider}}}} = props;
            return props.wizard.wizardType !== "tier" || !clusterId || !region || !provider;
        },
        options: (props) => {
            const {routingInfo: {urlParams: {decodedParams: {clusterId, region, provider}}}} = props;
            return {
                variables: {type: provider, clusterId, regionCode: region},
                fetchPolicy: "cache-first"
            };
        }
    })
)(stepinize(ReviewStep, {
    name: () => `Review`,
}));
