import _ from 'lodash'
import {
    CLUSTER_STATUS,
    DASHBOARD_TITLES,
    FREE_TRIAL, FREE_TRIAL_EXPIRED, FREE_TRIAL_WARNING,
    PRE_FREE_TRIAL, SHOW_TRIAL_INFO, SUBSCRIBED,
    WARNING_FREE_TRIAL_PERIOD
} from "./consts";
import numeral from "numeral";
import bytes from 'bytes'

export const defaultFilter = (item, filterString) => {
    if (_.isFunction(item)) return false;
    return _.values(item).find(value => {
        if (_.isEmpty(value) || !value) {
            return false;
        }
        else if(_.isObject(value)){
            return defaultFilter(value, filterString);
        }
        else if(_.isArray(value)){
            return false;
        }
        else return _.toString(value).toLowerCase().indexOf(filterString.toLowerCase()) !== -1;
    } );
};

export const defaultFilterWithKeys = (item, filterString, keys) => {
    if (_.isFunction(item)) return false;
    return _.keys(item).find(key => {
        if (!_.includes(keys, key)) return false; //not one of the keys to check
        const value = item[key];
        if (_.isEmpty(value) || !value) {
            return false;
        }
        else if(_.isObject(value)){
            return defaultFilterWithKeys(value, filterString, keys);
        }
        else if(_.isArray(value)){
            return false;
        }
        else return _.toString(value).toLowerCase().indexOf(filterString.toLowerCase()) !== -1;
    } );
};

export const calculateSavings = (cluster) => {
    const {status, tierInformation, capacityInformation} = cluster;
    const clusterIsOff = status && status.status !== CLUSTER_STATUS.ON;

    const currentSavingsCapacity = buildCapacityObject(_.get(tierInformation, "tieredCapacity", 0));

    const currentSavings = [];
    if (clusterIsOff) currentSavings.push({italicText: DASHBOARD_TITLES.dataNotAvailable});
    else if (currentSavingsCapacity.bytes > 0) {
        currentSavings.push({...currentSavingsCapacity});
        currentSavings.push({value: `$ ${numeral(_.get(tierInformation, "savings", 0)).format('0,0')}`});
    }
    else currentSavings.push({italicText: DASHBOARD_TITLES.notTiered});
    const savingOpportunityCapacity = buildCapacityObject(_.get(capacityInformation, "performanceCapacity.inactiveCapacity", 0));
    const additionalSavings = [];
    if (clusterIsOff) additionalSavings.push({italicText: DASHBOARD_TITLES.dataNotAvailable});
    else {
        const {volumes: {eligibleVolumes, tieredVolumes}} = cluster;
        if (savingOpportunityCapacity.bytes > 0) additionalSavings.push({...savingOpportunityCapacity});
        else if (tieredVolumes === 0) additionalSavings.push({italicText: DASHBOARD_TITLES.notTiered});
        else if (tieredVolumes && eligibleVolumes && tieredVolumes === eligibleVolumes) additionalSavings.push({italicText : DASHBOARD_TITLES.allTiered});
        else additionalSavings.push({italicText: DASHBOARD_TITLES.dataNotAvailable});
    }
    return {currentSavings, additionalSavings};
};

export const buildCapacityObject = (bytesInput) => {
    if (bytesInput === null) return bytesInput;
    const converted = bytes(bytesInput, {unitSeparator: " "});
    const convertedArray = converted ? converted.split(" ") : ["0", "B"];
    // console.log(`%c converted1 ${converted} convertedArray ${convertedArray}`, 'font-weight:bold; color:blue');
    return {
        bytes: bytesInput,
        value: convertedArray[0],
        unit: convertedArray[1]
    };
};

export const extractONTAPVersion = (versionString) => {
    const regex = /(\d+\.\d*|\d+)/;
    const version = versionString.match(regex)[0];
    return parseFloat(version);
};

export const extractError = (error, step) => {
    console.log(`%c ${step} Got GraphQL error ${JSON.stringify(error)}`, "font-weight:bold; color: red");
    if (error.networkError && error.networkError.result && error.networkError.result.errors) return error.networkError.result.errors[0].message;
    else if (error.graphQLErrors) return error.graphQLErrors[0].message;
    return `extarctError got UNKNOWN GraphQL error ${JSON.stringify(error)}`;
};

export const getAccountInfoStatus = (accountInfo) => {
    if (!accountInfo) return null;
    const daysLeftForTrial = _.get(accountInfo, "daysLeftForTrial", null);
    const status = {
        subscriptionStatus: accountInfo.subscriptionStatus,
        daysLeftForTrial
    };
    status.freeTrialStatus = daysLeftForTrial ? (daysLeftForTrial > WARNING_FREE_TRIAL_PERIOD ? FREE_TRIAL : WARNING_FREE_TRIAL_PERIOD) : PRE_FREE_TRIAL;
    if (_.isUndefined(daysLeftForTrial) || _.isNull(daysLeftForTrial)) {
        status.freeTrialStatus = PRE_FREE_TRIAL;
    }
    else if (daysLeftForTrial < 0) {
        status.freeTrialStatus = FREE_TRIAL_EXPIRED;
    }
    else if (daysLeftForTrial <= WARNING_FREE_TRIAL_PERIOD) {
        status.freeTrialStatus = FREE_TRIAL_WARNING;
    }
    else if (daysLeftForTrial > WARNING_FREE_TRIAL_PERIOD) {
        status.freeTrialStatus = FREE_TRIAL
    }
    return status;
};

export const extractByType = (object) => {
  return _.isArray(object) ? object[0] : object;
};

export const enforceLicense = (accountInfoStatus) => {
    return SHOW_TRIAL_INFO && accountInfoStatus && accountInfoStatus.freeTrialStatus === FREE_TRIAL_EXPIRED
        && accountInfoStatus.subscriptionStatus !== SUBSCRIBED;
};