import * as utils from '../../src/utils/utils'
import {DASHBOARD_TITLES} from "../../src/utils/consts";

describe('defaultFilter', () => {
    test('should return false when item is function', () => {
        expect(utils.defaultFilter(() => {
        })).toEqual(false);
    });
    test('should return false for an empty value in object', () => {
        expect(utils.defaultFilter({empty: null}, "exists")).toEqual(undefined);
    });
    test('should return the found object if the filter string exists', () => {
        expect(utils.defaultFilter({find: "found element"}, "element")).toEqual("found element");
    });
    test('should find the filter string even inside a compound object', () => {
        expect(utils.defaultFilter({compound: {find: "inner element"}}, "element")).toEqual({find: "inner element"});
    });
});

describe('defaultFilterWithKeys', function () {
    const item = {
        key1: "value1",
        key2: "value2",
        key3: "value3"
    };
    test('should return false if none of the requested keys exists in the object', () => {
        expect(utils.defaultFilterWithKeys(item, "value3", ["key1", "key2"])).toEqual(undefined);
    });
    test('should return the found key when filter string exists and the key is requested', () => {
        expect(utils.defaultFilterWithKeys(item, "value3", ["key3"])).toEqual("key3");
    });
    test('should return undefined when filter string does not exist even if thekey exists', () => {
        expect(utils.defaultFilterWithKeys(item, "not exist", ["key1", "key2", "key3"])).toEqual(undefined);
    });
});

describe('calculateSavings', function () {
    let cluster = {};
    beforeEach(() => {
        cluster = {
            agentCloudProvider: "AWS",
            agentId: null,
            capacityInformation: {
                performanceCapacity: {
                    activeCapacity: 4947802324992,
                    inactiveCapacity: 11544872091648,
                    isEstimatedInactiveCapacity: true,
                    usedPerformanceCapacity: 16492674416640
                },
                usedPerformanceAndTieredCapacity: 20340965113856
            },
            clusterId: "eda35168-d036-4cce-9cc8-ca916ec36326",
            name: "AFF-1",
            ontapClusterProperties: {
                clusterAddress: "192.168.0.1",
                ontapVersion: "9.4"
            },
            status: {
                message: null,
                status: "ON"
            },
            tierInformation: {
                bucketName: "bucket",
                objectStoreName: "tiering_objectStore",
                providerType: "AWS",
                regionName: "US East (Ohio)",
                savings: 16860,
                tierLevel: "Standard-Infrequent Access",
                tieredCapacity: 3848290697216
            },
            volumes: {
                eligibleVolumes: 16,
                tieredVolumes: 1
            }
        };
    });
    test('should return no available data when cluster is failed', () => {
        cluster.status = {message: "status is off", status: "FAILED"};
        expect(utils.calculateSavings(cluster)).toEqual({
            "additionalSavings": [{"italicText": "No Available Data"}],
            "currentSavings": [{"italicText": "No Available Data"}]
        });
    });

    test('should return no available data when cluster is unkown', () => {
        cluster.status = {message: "status is off", status: "UNKNOWN"};
        expect(utils.calculateSavings(cluster)).toEqual({
            "additionalSavings": [{"italicText": "No Available Data"}],
            "currentSavings": [{"italicText": "No Available Data"}]
        });
    });
    test('should build the right object when tieredCapacity = 0', () => {
        cluster.tierInformation.tieredCapacity = 0;
        expect(utils.calculateSavings(cluster)).toEqual({
            "additionalSavings": [{
                "bytes": 11544872091648,
                "unit": "TB",
                "value": "10.5"
            }], "currentSavings": [{"italicText": "Cold data has not been tiered yet"}]
        });
    });
    test('should build the right object when tieredCapacity > 0', () => {
        expect(utils.calculateSavings(cluster)).toEqual({
            "additionalSavings": [{
                "bytes": 11544872091648,
                "unit": "TB",
                "value": "10.5"
            }], "currentSavings": [{"bytes": 3848290697216, "unit": "TB", "value": "3.5"}, {"value": "$ 16,860"}]
        });
    });
    test('should build the right object when performanceCapacity.inactiveCapacity = 0', () => {
        cluster.capacityInformation.performanceCapacity.inactiveCapacity = 0;
        expect(utils.calculateSavings(cluster)).toEqual({
            "additionalSavings": [{"italicText": "No Available Data"}],
            "currentSavings": [{"bytes": 3848290697216, "unit": "TB", "value": "3.5"}, {"value": "$ 16,860"}]
        });
    });
    test('should build the right object when performanceCapacity.inactiveCapacity > 0', () => {
        expect(utils.calculateSavings(cluster)).toEqual({
            "additionalSavings": [{
                "bytes": 11544872091648,
                "unit": "TB",
                "value": "10.5"
            }], "currentSavings": [{"bytes": 3848290697216, "unit": "TB", "value": "3.5"}, {"value": "$ 16,860"}]
        });
    });
    test('should build the right object when tieredVolumes === 0', () => {
        cluster.capacityInformation.performanceCapacity.inactiveCapacity = 0;
        cluster.volumes.tieredVolumes = 0;
        expect(utils.calculateSavings(cluster).additionalSavings).toEqual([{"italicText": "Cold data has not been tiered yet"}]);
    });
    test('should build the right object when tieredVolumes === eligible volumes', () => {
        cluster.capacityInformation.performanceCapacity.inactiveCapacity = 0;
        cluster.volumes.tieredVolumes = 3;
        cluster.volumes.eligibleVolumes = 3;
        expect(utils.calculateSavings(cluster).additionalSavings).toEqual([{"italicText": "Data tiering is enabled on all volumes"}]);
    });

});